##############################################################################
#                                                                            #
#                  copyright (c) 2003 ITB, HU-Berlin                         #
#                  written by Raphael Ritz, r.ritz@biologie.hu-berlin.de     #
#                                                                            #
#                  extends 'PloneMinimalProduct'                             #
#                  copyright (c) 2003 Ingeniweb SARL                         #
#                                                                            #
##############################################################################

"""BibliographyEntry main class"""


from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
import string

from Products.CMFCore.CMFCorePermissions \
     import View, ManageProperties, ListFolderContents
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from AccessControl import Permissions, getSecurityManager, \
     ClassSecurityInfo, Unauthorized
from OFS import Folder

from global_symbols import *



# BibliographyEntry definition
factory_type_information = {
  'id'             : 'Bibliography Entry',
  'portal_type'    : 'Bibliography Entry',
  'meta_type'      : 'Bibliography Entry',
  'description'    : 'Base content type for handling references to \
  scientific publications',
  'content_icon'   : 'BibliographyEntry.gif',
  # Icon has to be acquired from a skin folder
  'product'        : 'CMFBibliography',
  'factory'        : 'addBibliographyEntry',
  'immediate_view' : 'BibliographyEntry_editForm',
  'filter_content_types' : 0,
  # This is for folderish types
  'allowed_content_types': (),
  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'BibliographyEntry_view',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'BibliographyEntry_editForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Metadata',
    'action'        : 'metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'bibtex',
    'name'         : 'BiBTeX',
    'action'       : 'bibtex',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    ),
  }
  
# The usual factory
def addBibliographyEntry(self, id, title = '', REQUEST = {}):
    """
    Factory method for a BibliographyEntry object
    """
    obj = BibliographyEntry(id, title=title)
    self._setObject(id, obj)

    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect('manage_main')



class BibliographyEntry(PortalContent, DefaultDublinCoreImpl):
    """BibliographyEntry class"""
    meta_type = factory_type_information['meta_type']
    # Has not to be the same as FTI but it is the most usual case.

    manage_options = PortalContent.manage_options

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)
    # $$$ Is this clever ? Isn't it better to make the object private ?

    def _setDefaults(self):
        """
        Set default attributes in current object
        for all properties in '_properties'
        """
        for prop in self._properties:
            if prop['type'] in ['text', 'string', 'selection']:
                setattr(self, prop['id'], '')
            elif prop['type'] in ['lines', 'tokens', 'multiple selection']:
                setattr(self, prop['id'], [])
            

    # Defaults
    authors = []

    # rr 2003/04/29 renamed 'month' to 'publication_month' 
    publication_month = ''  # for backwards compatibility after renaming
                            # from 'month' due to some clashes with the
                            # calendar tool

    def updateRenameMonth(self):
        """ update script for renaming of month property """
        if self.getProperty('month'):
            self.manage_addProperty('publication_month',
                                    self.getProperty('month'),
                                    'string')
        return None

    
    _properties=(
        {'id':'publication_type', 'type':'selection',
         'select_variable':'PublicationTypes', 'mode':'w'},
        {'id':'publication_title', 'type':'string', 'mode':'w'},
        {'id':'publication_url', 'type':'string', 'mode':'w'},
        {'id':'publication_year', 'type':'string', 'mode':'w'},
        {'id':'abstract', 'type':'text', 'mode':'w'},
        {'id':'publication_month', 'type':'string', 'mode':'w'},
        {'id':'journal', 'type':'string', 'mode':'w'},
        {'id':'volume', 'type':'string', 'mode':'w'},
        {'id':'number', 'type':'string', 'mode':'w'},
        {'id':'pages', 'type':'string', 'mode':'w'},
        {'id':'series', 'type':'string', 'mode':'w'},
        {'id':'edition', 'type':'string', 'mode':'w'},
        {'id':'editor', 'type':'string', 'mode':'w'},
        {'id':'publisher', 'type':'string', 'mode':'w'},
        {'id':'chapter', 'type':'string', 'mode':'w'},
        {'id':'booktitle', 'type':'string', 'mode':'w'},
        {'id':'note', 'type':'text', 'mode':'w'},
        {'id':'school', 'type':'string', 'mode':'w'},
        {'id':'institution', 'type':'string', 'mode':'w'},
        {'id':'address', 'type':'string', 'mode':'w'},
        {'id':'pmid', 'type':'string', 'mode':'w'},
        {'id':'preprint_server', 'type':'selection',
         'select_variable':'PreprintServers', 'mode':'w'},
        )

    _type_properties= {
        'standard':['publication_title', 'publication_year',
                    'publication_url', 'abstract'],
        'article':['journal', 'volume', 'number', 'pages'],
        'book':['editor', 'publisher', 'address', 'volume',
                'number', 'series', 'edition'],
        'incollection':['booktitle', 'editor', 'publisher',
                         'address', 'chapter', 'pages'],
        'inbook':['booktitle', 'editor', 'publisher',
                         'address', 'chapter', 'pages'],
        'inproceedings':['booktitle', 'editor', 'publisher',
                         'address', 'chapter', 'pages'],
        'masterthesis':['school','address'],
        'phdthesis':['school','address'],
        'preprint':['preprint_server'],
        'techreport':['institution', 'type', 'number', 'address'],
        'final':['publication_month', 'note']
        }

    def PublicationTypes(self):
        """ return the list of supported publication types """
        pts = ['article', 'book', 'booklet', 'inbook', 'incollection',
              'inproceedings', 'manual', 'masterthesis', 'misc',
              'phdthesis', 'preprint', 'proceedings', 'techreport',
              'unpublished', 'webpublished']
        return pts

    def PreprintServers(self):
        """list of known preprint servers"""
        return ['None',
                'arXiv',
                'CogPrints']

    def getPropertyOptions(self, id):
        """helper method for the editForm;
           returns the outcome of a call to the attribute/method
           hold in the select_varible of the property specified by
           the passed in 'id' """
        if self.getPropertyType(id) not in ['selection', 'multiple selection']:
            return None
        sel_var = None
        for prop in self.propertyMap():
            if prop['id'] == id:
                sel_var = prop['select_variable']
                break
        if sel_var:
            raw = getattr(self, sel_var)
            if type(raw) == type([]): return raw
            else: return raw()
        else:
            return []

    def getTypeProperties(self):
        """ editable properties for this publication type """
        pt = self.getProperty('publication_type', '')
        if not pt:
            return None
        else:
            special = self.getSpecialProperties(pt)
            return self._type_properties['standard'] + special \
                   + self._type_properties['final']

    def getSpecialProperties(self,type):
        """ the properties special to this type"""
        if self._type_properties.has_key(type):
            return self._type_properties[type]
        else:
            return []

    def getURL(self, defaultURL=None):
        """the publication_url if set, otherwise a link to PubMed
           if pmid is set or the default otherwise"""
        if self.getProperty('publication_url'):
            return self.publication_url
        elif self.getProperty('pmid'):
            url = "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Retrieve"
            return url + "&db=PubMed&list_uids=%s" % self.pmid
        else: return defaultURL


    # some stuff for author handling

    def getAuthorList(self):
        """a list of dictonaries, one for each author"""
        return self.authors

    security.declareProtected(BibliographyEntry_editPermission, 'setAuthors')
    def setAuthors(self, authors=[]):
        """pass in a list of dictonaries; one for each author"""
        if authors:
            self.authors = authors
        return None

    security.declareProtected(BibliographyEntry_editPermission, 'addAuthor')
    def addAuthor(self, author={}):
        """pass in a dictonary with keys:
           'firstname', 'middlename', 'lastname', 'homepage'"""
        if author:
            self.authors.append(author)
        return None

    def AuthorURLs(self):
        """a string with all the known author's URLs;
           helper method for bibtex output"""
        a_list = self.getAuthorList()
        a_URLs = ''
        for a in a_list:
            url = a.get('homepage', ' ')
            a_URLs += "%s and " % url
        return a_URLs[:-5]

    def Authors(self, sep=', ', lastsep=', and ',
                abbrev=1, lastnamefirst=1, withURL=0):
        """string formated author list.
        
           Arguments:

             - sep: specifies the separator between authors
                   (exept for the last)
                    default: ', '
                    
             - lastsep: the last separator;
                    default: ', and ' (the ',' is removed if there
                    are only two authors)
                    
             - abbrev: if true, first and middle names are abbreviated
                    default: 1

             - lastnamefirst: if true, the ordering is
                    lastname, firstname, middlename
                    otherwise it is firstname, middlename, lastname
                    default: 1

             - withURL: whether or not to embed the authorname in an
                    anchor tag pointing to his homepage (if present)
                    default: 0
        """
        authors = self.authors
        if not authors:
            return "No authors specified"

        nofa = len(authors)

        if nofa == 1:
            return self.formatAuthor(authors[0],abbrev,
                                     lastnamefirst,withURL)

        if nofa == 2:
            if ',' in lastsep and not abbrev:
                lastsep = lastsep.replace(',','')

            a1 = self.formatAuthor(authors[0],abbrev,
                                   lastnamefirst,withURL)
            a2 = self.formatAuthor(authors[1],abbrev,
                                   lastnamefirst,withURL)

            return a1 + lastsep + a2
        
        result = ""
        for author in authors[:-2]:
            result += self.formatAuthor(author,abbrev,
                                        lastnamefirst,withURL)\
                      + sep
        result += self.formatAuthor(authors[-2],abbrev,
                                    lastnamefirst,withURL)\
                  + lastsep
        result += self.formatAuthor(authors[-1],abbrev,
                                    lastnamefirst,withURL)

        return result

    def formatAuthor(self,author,abbrev,lastnamefirst,withURL):
        """string formats an individual author"""

        fname = author.get('firstname', '')
        mname = author.get('middlename', '')
        lname = author.get('lastname', '')
        homepage = author.get('homepage', '')
        astring = ''

        if abbrev:
            if fname: fname = fname[0]
            if mname: mname = mname[0]

        if lastnamefirst:
            if abbrev:
                astring = "%s, %s%s" % (lname, fname, mname)
            else:
                astring = "%s, %s %s" % (lname, fname, mname)
                # remove the trailing blank if mname is empty
                if not mname: astring = astring.strip()
        else:
            if abbrev:
                astring = "%s%s %s" % (fname, mname, lname)
            else:
                astring = "%s %s %s" % (fname, mname, lname)
                # remove double blanks (happens if mname is empty)
                astring = astring.replace('  ', ' ')

        if withURL and homepage:
            return """<a href="%s">%s</a>""" % (homepage, astring)
        else:
            return astring
            
    def Source(self):
        """something like: journal, volume, number, pages
           or: In: booktitle, edited by someone ..."""

        pt = self.publication_type
        c  = ""

        if pt == 'article':
            c = "%s %s(%s):%s" % (self.journal, self.volume,
                                  self.number, self.pages)
            # remove '()' (happens if number is empty)
            c = c.replace('()','')

        elif pt == 'book':
            c = "%s, %s" % (self.publisher, self.address)
            if c[-2] == ',': c = c[:-2]

        elif pt in ['inbook','incollection', 'inproceedings']:
            c = "In: %s. Edited by %s. %s %s" % (self.booktitle,
                                                 self.editor,
                                                 self.publisher,
                                                 self.address)

        elif pt in ['masterthesis','phdthesis']:
            c = "%s, %s, %s" % (pt.capitalize(), self.school, self.address)

        elif pt in ['preprint']:
            hosted = ''
            if self.preprint_server and self.preprint_server != 'None':
                hosted = " at %s" % self.preprint_server
            c = "%s%s" % (pt.capitalize(), hosted)

        else:
            c = "%s" % (pt.capitalize(), )

        return c.strip() + '.'

    def bibtex(self):
        """returns the entry in BiBTeX format"""
        bibtex = "\n@%s{%s," % (self.publication_type, self.getId())
        bibtex += "\n  author = {%s}," % self.Authors(sep=' and ',
                                                      lastsep=' and ',
                                                      abbrev=0,
                                                      lastnamefirst=0)
        aURLs = self.AuthorURLs()
        if aURLs.find('http') > -1:
            bibtex += "\n  authorURLs = {%s}," % aURLs
        bibtex += "\n  title = {%s}," % self.publication_title
        bibtex += "\n  year = {%s}," % self.publication_year
        bibtex += "\n  abstract = {%s}," % self.abstract
        for prop in self.getSpecialProperties(self.publication_type):
            value = self.getProperty(prop)
            if value:
                bibtex += "\n  %s = {%s}," % (prop, value)
        kws = ', '.join(self.Subject())
        if kws:
            bibtex += "\n  keywords = {%s}," % kws
        bibtex += "\n  note = {%s}" % self.note
        bibtex += "\n}\n"

        return bibtex
    

    def SearchableText(self):
        """returns all text relevant to full text search """
        result = [self.Authors(abbrev=0), ', '.join(self.Subject())]
        for prop in self._properties:
            if prop['type'] in ['string', 'text', 'selection']:
                result.append(getattr(self, prop['id']))
        return ' '.join(result)
        
    
    # Init method
    def __init__(self, id, **kw):
        """__init__(self, id, **kw)"""

        self.id = id
        self._setDefaults()
        DefaultDublinCoreImpl.__init__(self)

    # Class-internal factory method
    security.declareProtected(BibliographyEntry_addPermission,
                              'manage_addBibliographyEntry')
    def manage_addBibliographyEntry(self, id, title='', REQUEST=None):
        """ adds a new PloneFolder """
        ob=BibliographyEntry(id, title)
    	self._setObject(id, ob)
    	if REQUEST is not None:
            return self.folder_contents(self, REQUEST,
                                        portal_status_message='Added content.')


    # Edit method (change this to suit your needs)
    # This edit method should only change attributes that are
    # neither 'id' or metadatas.
    security.declareProtected(BibliographyEntry_editPermission,
                              'manage_editBibliographyEntry')
    def manage_editBibliographyEntry(self, REQUEST=None, propdict=None, **kw):
        """
        manage_editBibliographyEntry(self, REQUEST=None, **kw)
        => object modification method
        """
        if REQUEST is not None:
            kw.update(REQUEST.form)
        if propdict is not None:
            kw.update(propdict)
            
        apply(self.manage_changeProperties, (), kw)

        if kw.has_key('authors'):
            self.setAuthors(kw['authors'])
        if kw.has_key('nextauthor'):
            self.addAuthor(kw['nextauthor'])

        ptitle = kw.get('publication_title', '')
        text = kw.get('abstract', '')
        keywords = kw.get('keywords', [])  # uploads my need this

        if ptitle: self.setTitle(ptitle)
        if text: self.setDescription(text)
        if keywords: self.setSubject(keywords)

        self.reindexObject()

        #
        return None
        # No need for a Redirect here
    	# if REQUEST is not None:
        #     psm = 'Updated content.'
        #     return self.folder_contents(self, REQUEST,
        #                                 portal_status_message=psm)
        



manage_addBibliographyEntry = BibliographyEntry.manage_addBibliographyEntry


# Class instanciation
InitializeClass(BibliographyEntry)

