##############################################################################
#                                                                            #
#               copyright (c) 2003 ITB, Humboldt-University Berlin           #
#               written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                            #
#               buildt by extending the                                      #
#               PloneMinimalProdukt                                          #
#               copyright (c) 2003 Ingeniweb SARL                            #
#                                                                            #
##############################################################################

"""BibliographyFolder main class"""


from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
import string, re

from Products.CMFCore.CMFCorePermissions import View, ManageProperties
from Products.CMFCore.CMFCorePermissions import ListFolderContents
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFDefault import SkinnedFolder
from AccessControl import Permissions, getSecurityManager
from AccessControl import ClassSecurityInfo, Unauthorized
from OFS import Folder

from global_symbols import *

# the parsers need this
medline_splitter = re.compile('UI  - ')
medline_pattern = re.compile(r'(^.{0,4}- )', re.M)

bibtex_splitter = re.compile('}\s*@')
bibtex_pattern = re.compile('(,\s*\w{2,}\s*=)')



# BibliographyFolder definition
factory_type_information = {
  'id'             : 'Bibliography Folder',
  'portal_type'    : 'Bibliography Folder',
  'meta_type'      : 'Bibliography Folder',
  'description'    : 'A folderish type for managing publication references',
  'content_icon'   : 'BibliographyFolder.gif',
  # Icon has to be acquired from a skin folder
  'product'        : 'CMFBibliography',
  'factory'        : 'addBibliographyFolder',
  'immediate_view' : 'folder_listing',                          
  'filter_content_types' : 0,
  # This is for folderish types
  'allowed_content_types': (),
  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'BibliographyFolder_view',
    'permissions'  : (CMFCorePermissions.View, ),
##    'category'      : 'folder',
    },
    {
    'id'            : 'folderlisting',
    'name'          : 'Folder Listing',
    'action'        : 'folder_listing',
    'permissions'   : (Permissions.access_contents_information,),
    'category'      : 'folder',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'BibliographyFolder_editForm',
    'permissions'  : (BibliographyFolder_editPermission,),
##    'category'      : 'folder',
    },
    {
    'id'           : 'upload',
    'name'         : 'Upload',
    'action'       : 'uploadForm',
    'permissions'  : (BibliographyFolder_editPermission,),
##    'category'      : 'folder',
    },
  ),
  }

# The usual factory
def addBibliographyFolder(self, id, title = '', REQUEST = {}):
    """
    Factory method for a BibliographyFolder object
    """
    obj = BibliographyFolder(id, title)
    self._setObject(id, obj)

    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect('manage_main')





class BibliographyFolder(SkinnedFolder.SkinnedFolder, ):
    """BibliographyFolder class"""
    meta_type = factory_type_information['meta_type']
    # Has not to be the same as FTI but it is the most usual case.

    manage_options = Folder.Folder.manage_options

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)
    # $$$ Is this clever ? Isn't it better to make the object private ?


    # Dummy permission instanciation.
    # Permissions has to be "dummily" instanciated in order to be reachable
    # for the factory_type_information structure.
    # In order to be very clear, we just declare each and every
    # permission we use in this product.
    security.declareProtected(BibliographyFolder_editPermission,
                              "dummyMethod_editPermission")
    security.declareProtected(BibliographyFolder_addPermission,
                              "dummyMethod_addPermission")

    # Init method
    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""

        self.id = id
        self.title = title

    # Class-internal factory method
    security.declareProtected(BibliographyFolder_addPermission,
                              'manage_addBibliographyFolder')
    def manage_addBibliographyFolder(self, id, title='', REQUEST=None):
        """ adds a new PloneFolder """
        ob=PloneFolder(id, title)
    	self._setObject(id, ob)
    	if REQUEST is not None:
            return self.folder_contents(self, REQUEST,
                                        portal_status_message='Added folder.')

    # Edit method (change this to suit your needs)
    # This edit method should only change attributes
    # that are neither 'id' or metadatas.
    security.declareProtected(BibliographyFolder_editPermission,
                              'manage_editBibliographyFolder')
    def manage_editBibliographyFolder(self, title = '', REQUEST = {}):
        """
        manage_editBibliographyFolder(self, title = '', REQUEST = {})
        => object modification method
        """
        # Change title
        self.title = title
    	if REQUEST is not None:
            return self.folder_contents(self, REQUEST,
                                        portal_status_message='Updated folder.')
        


    # This method comes from PloneFolder class.
    security.declareProtected(Permissions.access_contents_information,
                              'listFolderContents')
    def listFolderContents( self, spec=None, contentFilter=None,
                            suppressHiddenFiles=0 ): 
        """
        Hook around 'contentValues' to let 'folder_contents'
        be protected.  Duplicating skip_unauthorized behavior of dtml-in.
        
        In the world of Plone we do not want to show objects that begin with a .
        So we have added a simply check.  We probably dont want to raise an
        Exception as much as we want to not show it.
        
        """

        items = self.contentValues(spec=spec, filter=contentFilter)
        l = []
        for obj in items:
            id = obj.getId()
            v = obj
            try:
                if suppressHiddenFiles and id[:1]=='.': 
                    raise Unauthorized(id, v)
                if getSecurityManager().validate(self, self, id, v):
                    l.append(obj)
            except (Unauthorized, 'Unauthorized'):
                pass
        return l


    #
    # Extensions for Publication Folder
    #

    def processUpload(self, source, filename=''):
        """ main routine to be called for uploading """

        format  = self._guessFormat(source, filename)
        entries = self.splitSource(source, format)

        for entry in entries:
            reference = self.parse(entry, format)
            newid = self.cookId(reference)
            if newid and newid != "nobody1000":
                self.invokeFactory('Bibliography Entry', newid)
                obj = getattr(self, newid)
                obj.manage_editBibliographyEntry(None,reference)
        return None

    def _guessFormat(self, source, filename=''):
        
        supported = {'med':'medline'
                    ,'bib':'bibtex'
                     }
        
        if filename:
            suffix = filename.split('.')[-1]
            if supported.has_key(suffix):
                return supported[suffix]
        else:
            return "Could not infer format"

    def splitSource(self, source, format):
        """returns a list of the entries from a file;
           format depended splitter"""
        entries = []
        
        if format == 'medline':
            entries = medline_splitter.split(source)
        elif format == 'bibtex':
            entries = bibtex_splitter.split(source)
        else: return None
        return entries

    def cookId(self, ref):
        """ Come up with a unique id for this folder"""
        newid = ''
        if ref.has_key('pid'):
            newid = ref['pid']
        elif ref.has_key('authors'):
            namepart = ref['authors'][0]['lastname']
        else:
            namepart="nobody"

        if ref.has_key('publication_year') and ref['publication_year']:
            yearpart = str(ref['publication_year'])
        else:
            yearpart = "1000"

        if not newid: newid = namepart + yearpart

        while newid in self.contentIds(): newid = self.nextId(newid)

        return newid

    def nextId(self, testid):
        letters = string.letters
        if testid[-1] in letters:
            last = lewtters[letters.find(testid[-1])+1]
            return testid[:-1] + last
        else:
            return testid + 'a'

    def group(self, p,n):
        """ Group a sequence p into a list of n tuples."""
        mlen, lft = divmod(len(p), n)
        if lft != 0: mlen += 1

        # initialize a list of suitable length
        lst = [[None]*n for i in range(mlen)]
        
        # Loop over all items in the input sequence
        for i in range(len(p)):
            j,k = divmod(i,n)
            lst[j][k] = p[i]

        return map(tuple, lst)
    
    
    def parse(self, entry='', format=None):
        """parse a publication reference"""
        
        supported = ['medline', 'bibtex']
        
        if format not in supported: return "%s-format not supported" % format
        elif format == 'medline': return self.parse_medline(entry)
        elif format == 'bibtex': return self.parse_bibtex(entry)
        

    def parse_bibtex(self, entry=''):
        """parse a publication reference in bibtex format"""

        result = {}
        authorlist = authorURLlist = []

        tokens = bibtex_pattern.split(entry)
    
        try: 
            type, pid = tokens[0].strip().split('{')
            result['publication_type'] = type.replace('@', '').strip().lower()
            result['pid'] = pid.replace(',', '').strip()
        except:
            return "Parser Error: malformed first line."

        for k,v in self.group(tokens[1:],2):
            key = k[1:-1].strip()
            v = v[:].strip()
        
            if v[0] in ['{', '"']:
                value = v[1:-1].strip()
            else:
                value = v

            result[key] = value


        # compile authors list of dictionaries
        if result.has_key('author'):
            authorlist = result['author'].split(' and')
        if result.has_key('authorURLs'):
            authorURLlist = result['authorURLs'].split('and ')

        if authorlist:
            alist = []
            for author in authorlist:
                fname = mname = lname = ''
                parts = author.split()
                if len(parts) == 1:
                    lname = parts[0].strip()
                else:
                    lname = parts[-1].strip()
                    fname = parts[0].strip()
                    if parts[1:-1]:
                        for part in parts[1:-1]:
                            mname = mname + part.strip()
                adict = {'firstname': fname,
                         'middlename': mname,
                         'lastname': lname}
                alist.append(adict)

        if authorURLlist and alist:
            index = 0
            for url in authorURLlist:
                alist[index]['homepage'] = url.strip()
                index += 1

        if authorlist:
            result['authors'] = alist

        # do some renaming and reformatting
        tmp = result.get('note')
        while tmp and tmp[-1] in ['}', ',', '\n', '\r']:
            tmp = tmp[:-1]
        result['note'] = tmp
        # make keywords a list
        try:
            tmp = eval(result.get('keywords', '[]'))
        except:
            tmp = result.get('keywords').split(',')
        result['keywords'] = tmp
        result['publication_year'] = result.get('year', '')
        result['publication_month'] = result.get('month', '')
        result['publication_url'] = result.get('URL', '')
        result['publication_title'] = result.get('title', '')
    
        return result


    def parse_medline(self, entry=''):
        """parse a publication reference in medline format"""

        tokens = medline_pattern.split(entry)    
    
        nested = []
        for i in range(1,len(tokens),2):
            nested.append([tokens[i],tokens[i+1]])

        result = {}
        # some defaults
        result['note'] = 'automatic medline import'

        for key, value in nested:
            if key == 'PT  - ' and value.find('Journal Article')> -1:
                result['publication_type'] = 'article'
            elif key == 'TI  - ':
                title = value.replace('\n', ' ').replace('      ', '').strip()
                result['publication_title'] = title
            elif key == 'AB  - ':
                tmp = value.replace('\n', ' ').replace('  ', '')
                result['abstract'] = tmp.replace('  ', '').replace('  ', '')
            elif key == 'PMID- ': result['pmid'] = str(value).strip()
            elif key == 'TA  - ': result['journal'] = str(value).strip()
            elif key == 'VI  - ': result['volume'] = str(value).strip()
            elif key == 'IP  - ': result['number'] = str(value).strip()
            elif key == 'PG  - ': result['pages'] = str(value).strip()
            elif key == 'DP  - ':
                result['publication_year'] = value[:4]
                result['publication_month'] = value[5:]
            elif key == 'FAU - ':
                raw = value.replace('\n', '').split(', ')
                lname = raw[0]
                fnames = raw[1].split(' ',1)
                fname = fnames[0]
                if len(fnames)> 1:
                    minit = fnames[1]
                else:
                    minit = ''
                # alist = [fname, minit, lname, '']
                adict = {'firstname': fname
                         ,'middlename': minit
                         ,'lastname': lname
                         }
                result.setdefault('authors',[]).append(adict)
        return result


manage_addBibliographyFolder = BibliographyFolder.manage_addBibliographyFolder


# Class instanciation
InitializeClass(BibliographyFolder)


